@@ -1,4 +1,5 @@
-# Copyright 2013-2020 CERN for the benefit of the ATLAS collaboration.
+# -*- coding: utf-8 -*-
+# Copyright 2013-2020 CERN
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -19,7 +20,7 @@
 # - Cedric Serfon <cedric.serfon@cern.ch>, 2014-2018
 # - Thomas Beermann <thomas.beermann@cern.ch>, 2014
 # - Martin Barisits <martin.barisits@cern.ch>, 2015-2019
-# - Joaquin Bogado <jbogado@linti.unlp.edu.ar>, 2018
+# - Joaquín Bogado <jbogado@linti.unlp.edu.ar>, 2018
 # - Dimitrios Christidis <dimitrios.christidis@cern.ch>, 2018
 # - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2019
 # - Tobias Wegner <twegner@cern.ch>, 2019
@@ -39,7 +40,6 @@ from json import dumps, loads
 
 import pytest
 import xmltodict
-from paste.fixture import TestApp
 
 from rucio.client.baseclient import BaseClient
 from rucio.client.didclient import DIDClient
@@ -47,7 +47,7 @@ from rucio.client.replicaclient import ReplicaClient
 from rucio.client.ruleclient import RuleClient
 from rucio.common.config import config_get, config_get_bool
 from rucio.common.exception import (DataIdentifierNotFound, AccessDenied, UnsupportedOperation,
-                                    RucioException, ReplicaIsLocked, ReplicaNotFound)
+                                    RucioException, ReplicaIsLocked, ReplicaNotFound, ScopeNotFound)
 from rucio.common.types import InternalAccount, InternalScope
 from rucio.common.utils import generate_uuid, clean_surls
 from rucio.core.did import add_did, attach_dids, get_did, set_status, list_files, get_did_atime
@@ -62,9 +62,7 @@ from rucio.daemons.badreplicas.minos_temporary_expiration import run as minos_te
 from rucio.daemons.badreplicas.necromancer import run as necromancer_run
 from rucio.db.sqla.constants import DIDType, ReplicaState, OBSOLETE
 from rucio.rse import rsemanager as rsemgr
-from rucio.tests.common import execute, rse_name_generator
-from rucio.web.rest.authentication import APP as auth_app
-from rucio.web.rest.replica import APP as rep_app
+from rucio.tests.common import execute, rse_name_generator, headers, auth
 
 
 class TestReplicaCore(unittest.TestCase):
@@ -575,365 +573,343 @@ class TestReplicaCore(unittest.TestCase):
         assert cov[mock4] == 500
 
 
-class TestReplicaClients(unittest.TestCase):
+def test_add_list_bad_replicas(vo, replica_client, did_client):
+    """ REPLICA (CLIENT): Add bad replicas"""
+    tmp_scope = 'mock'
+    nbfiles = 5
+    # Adding replicas to deterministic RSE
+    files = [{'scope': tmp_scope, 'name': 'file_%s' % generate_uuid(), 'bytes': 1, 'adler32': '0cc737eb', 'meta': {'events': 10}} for _ in range(nbfiles)]
+    rse_id1 = get_rse_id('MOCK', vo=vo)
+    replica_client.add_replicas(rse='MOCK', files=files)
 
-    def setUp(self):
-        if config_get_bool('common', 'multi_vo', raise_exception=False, default=False):
-            self.vo = {'vo': config_get('client', 'vo', raise_exception=False, default='tst')}
-            self.vo_header = {'X-Rucio-VO': self.vo['vo']}
-        else:
-            self.vo = {}
-            self.vo_header = {}
+    # Listing replicas on deterministic RSE
+    replicas, list_rep = [], []
+    for replica in replica_client.list_replicas(dids=[{'scope': f['scope'], 'name': f['name']} for f in files], schemes=['srm'], unavailable=True):
+        replicas.extend(replica['rses']['MOCK'])
+        list_rep.append(replica)
+    r = replica_client.declare_bad_file_replicas(replicas, 'This is a good reason')
+    assert r == {}
+    bad_replicas = list_bad_replicas()
+    nbbadrep = 0
+    for rep in list_rep:
+        for badrep in bad_replicas:
+            if badrep['rse_id'] == rse_id1:
+                if badrep['scope'].external == rep['scope'] and badrep['name'] == rep['name']:
+                    nbbadrep += 1
+    assert len(replicas) == nbbadrep
 
-        self.replica_client = ReplicaClient()
-        self.did_client = DIDClient()
+    # Run necromancer once
+    necromancer_run(threads=1, bulk=10000, once=True)
 
-    def test_add_list_bad_replicas(self):
-        """ REPLICA (CLIENT): Add bad replicas"""
-        tmp_scope = 'mock'
-        nbfiles = 5
-        # Adding replicas to deterministic RSE
-        files = [{'scope': tmp_scope, 'name': 'file_%s' % generate_uuid(), 'bytes': 1, 'adler32': '0cc737eb', 'meta': {'events': 10}} for _ in range(nbfiles)]
-        rse_id1 = get_rse_id('MOCK', **self.vo)
-        self.replica_client.add_replicas(rse='MOCK', files=files)
+    # Try to attach a lost file
+    tmp_dsn = 'dataset_%s' % generate_uuid()
+    did_client.add_dataset(scope=tmp_scope, name=tmp_dsn)
+    with pytest.raises(UnsupportedOperation):
+        did_client.add_files_to_dataset(tmp_scope, name=tmp_dsn, files=files, rse='MOCK')
 
-        # Listing replicas on deterministic RSE
-        replicas, list_rep = [], []
-        for replica in self.replica_client.list_replicas(dids=[{'scope': f['scope'], 'name': f['name']} for f in files], schemes=['srm'], unavailable=True):
-            replicas.extend(replica['rses']['MOCK'])
-            list_rep.append(replica)
-        r = self.replica_client.declare_bad_file_replicas(replicas, 'This is a good reason')
-        assert r == {}
-        bad_replicas = list_bad_replicas()
-        nbbadrep = 0
-        for rep in list_rep:
-            for badrep in bad_replicas:
-                if badrep['rse_id'] == rse_id1:
-                    if badrep['scope'].external == rep['scope'] and badrep['name'] == rep['name']:
-                        nbbadrep += 1
-        assert len(replicas) == nbbadrep
+    # Adding replicas to non-deterministic RSE
+    files = [{'scope': tmp_scope, 'name': 'file_%s' % generate_uuid(), 'bytes': 1, 'adler32': '0cc737eb',
+              'pfn': 'srm://mock2.com:8443/srm/managerv2?SFN=/rucio/tmpdisk/rucio_tests/%s/%s' % (tmp_scope, generate_uuid()), 'meta': {'events': 10}} for _ in range(nbfiles)]
+    rse_id2 = get_rse_id('MOCK2', vo=vo)
+    replica_client.add_replicas(rse='MOCK2', files=files)
 
-        # Run necromancer once
-        necromancer_run(threads=1, bulk=10000, once=True)
+    # Listing replicas on non-deterministic RSE
+    replicas, list_rep = [], []
+    for replica in replica_client.list_replicas(dids=[{'scope': f['scope'], 'name': f['name']} for f in files], schemes=['srm'], unavailable=True):
+        replicas.extend(replica['rses']['MOCK2'])
+        list_rep.append(replica)
+    print(replicas, list_rep)
+    r = replica_client.declare_bad_file_replicas(replicas, 'This is a good reason')
+    print(r)
+    assert r == {}
+    bad_replicas = list_bad_replicas()
+    nbbadrep = 0
+    for rep in list_rep:
+        for badrep in bad_replicas:
+            if badrep['rse_id'] == rse_id2:
+                if badrep['scope'].external == rep['scope'] and badrep['name'] == rep['name']:
+                    nbbadrep += 1
+    assert len(replicas) == nbbadrep
 
-        # Try to attach a lost file
-        tmp_dsn = 'dataset_%s' % generate_uuid()
-        self.did_client.add_dataset(scope=tmp_scope, name=tmp_dsn)
-        with pytest.raises(UnsupportedOperation):
-            self.did_client.add_files_to_dataset(tmp_scope, name=tmp_dsn, files=files, rse='MOCK')
+    # Now adding non-existing bad replicas
+    files = ['srm://mock2.com/rucio/tmpdisk/rucio_tests/%s/%s' % (tmp_scope, generate_uuid()), ]
+    r = replica_client.declare_bad_file_replicas(files, 'This is a good reason')
+    output = ['%s Unknown replica' % rep for rep in files]
+    assert r == {'MOCK2': output}
 
-        # Adding replicas to non-deterministic RSE
-        files = [{'scope': tmp_scope, 'name': 'file_%s' % generate_uuid(), 'bytes': 1, 'adler32': '0cc737eb',
-                  'pfn': 'srm://mock2.com:8443/srm/managerv2?SFN=/rucio/tmpdisk/rucio_tests/%s/%s' % (tmp_scope, generate_uuid()), 'meta': {'events': 10}} for _ in range(nbfiles)]
-        rse_id2 = get_rse_id('MOCK2', **self.vo)
-        self.replica_client.add_replicas(rse='MOCK2', files=files)
 
-        # Listing replicas on non-deterministic RSE
-        replicas, list_rep = [], []
-        for replica in self.replica_client.list_replicas(dids=[{'scope': f['scope'], 'name': f['name']} for f in files], schemes=['srm'], unavailable=True):
-            replicas.extend(replica['rses']['MOCK2'])
-            list_rep.append(replica)
-        print(replicas, list_rep)
-        r = self.replica_client.declare_bad_file_replicas(replicas, 'This is a good reason')
-        print(r)
-        assert r == {}
-        bad_replicas = list_bad_replicas()
-        nbbadrep = 0
-        for rep in list_rep:
-            for badrep in bad_replicas:
-                if badrep['rse_id'] == rse_id2:
-                    if badrep['scope'].external == rep['scope'] and badrep['name'] == rep['name']:
-                        nbbadrep += 1
-        assert len(replicas) == nbbadrep
+def test_add_suspicious_replicas(replica_client):
+    """ REPLICA (CLIENT): Add suspicious replicas"""
+    tmp_scope = 'mock'
+    nbfiles = 5
+    # Adding replicas to deterministic RSE
+    files = [{'scope': tmp_scope, 'name': 'file_%s' % generate_uuid(), 'bytes': 1, 'adler32': '0cc737eb', 'meta': {'events': 10}} for _ in range(nbfiles)]
+    replica_client.add_replicas(rse='MOCK', files=files)
 
-        # Now adding non-existing bad replicas
-        files = ['srm://mock2.com/rucio/tmpdisk/rucio_tests/%s/%s' % (tmp_scope, generate_uuid()), ]
-        r = self.replica_client.declare_bad_file_replicas(files, 'This is a good reason')
-        output = ['%s Unknown replica' % rep for rep in files]
-        assert r == {'MOCK2': output}
+    # Listing replicas on deterministic RSE
+    replicas = []
+    list_rep = []
+    for replica in replica_client.list_replicas(dids=[{'scope': f['scope'], 'name': f['name']} for f in files], schemes=['srm'], unavailable=True):
+        replicas.extend(replica['rses']['MOCK'])
+        list_rep.append(replica)
+    r = replica_client.declare_suspicious_file_replicas(replicas, 'This is a good reason')
+    assert r == {}
+    # Adding replicas to non-deterministic RSE
+    files = [{'scope': tmp_scope, 'name': 'file_%s' % generate_uuid(), 'bytes': 1, 'adler32': '0cc737eb',
+              'pfn': 'srm://mock2.com:8443/srm/managerv2?SFN=/rucio/tmpdisk/rucio_tests/%s/%s' % (tmp_scope, generate_uuid()), 'meta': {'events': 10}} for _ in range(nbfiles)]
+    replica_client.add_replicas(rse='MOCK2', files=files)
 
-    def test_add_suspicious_replicas(self):
-        """ REPLICA (CLIENT): Add suspicious replicas"""
-        tmp_scope = 'mock'
-        nbfiles = 5
-        # Adding replicas to deterministic RSE
-        files = [{'scope': tmp_scope, 'name': 'file_%s' % generate_uuid(), 'bytes': 1, 'adler32': '0cc737eb', 'meta': {'events': 10}} for _ in range(nbfiles)]
-        self.replica_client.add_replicas(rse='MOCK', files=files)
+    # Listing replicas on non-deterministic RSE
+    replicas = []
+    list_rep = []
+    for replica in replica_client.list_replicas(dids=[{'scope': f['scope'], 'name': f['name']} for f in files], schemes=['srm'], unavailable=True):
+        replicas.extend(replica['rses']['MOCK2'])
+        list_rep.append(replica)
+    r = replica_client.declare_suspicious_file_replicas(replicas, 'This is a good reason')
+    assert r == {}
+    # Now adding non-existing bad replicas
+    files = ['srm://mock2.com/rucio/tmpdisk/rucio_tests/%s/%s' % (tmp_scope, generate_uuid()), ]
+    r = replica_client.declare_suspicious_file_replicas(files, 'This is a good reason')
+    output = ['%s Unknown replica' % rep for rep in files]
+    assert r == {'MOCK2': output}
 
-        # Listing replicas on deterministic RSE
-        replicas = []
-        list_rep = []
-        for replica in self.replica_client.list_replicas(dids=[{'scope': f['scope'], 'name': f['name']} for f in files], schemes=['srm'], unavailable=True):
-            replicas.extend(replica['rses']['MOCK'])
-            list_rep.append(replica)
-        r = self.replica_client.declare_suspicious_file_replicas(replicas, 'This is a good reason')
-        assert r == {}
-        # Adding replicas to non-deterministic RSE
-        files = [{'scope': tmp_scope, 'name': 'file_%s' % generate_uuid(), 'bytes': 1, 'adler32': '0cc737eb',
-                  'pfn': 'srm://mock2.com:8443/srm/managerv2?SFN=/rucio/tmpdisk/rucio_tests/%s/%s' % (tmp_scope, generate_uuid()), 'meta': {'events': 10}} for _ in range(nbfiles)]
-        self.replica_client.add_replicas(rse='MOCK2', files=files)
 
-        # Listing replicas on non-deterministic RSE
-        replicas = []
-        list_rep = []
-        for replica in self.replica_client.list_replicas(dids=[{'scope': f['scope'], 'name': f['name']} for f in files], schemes=['srm'], unavailable=True):
-            replicas.extend(replica['rses']['MOCK2'])
-            list_rep.append(replica)
-        r = self.replica_client.declare_suspicious_file_replicas(replicas, 'This is a good reason')
-        assert r == {}
-        # Now adding non-existing bad replicas
-        files = ['srm://mock2.com/rucio/tmpdisk/rucio_tests/%s/%s' % (tmp_scope, generate_uuid()), ]
-        r = self.replica_client.declare_suspicious_file_replicas(files, 'This is a good reason')
-        output = ['%s Unknown replica' % rep for rep in files]
-        assert r == {'MOCK2': output}
+def test_bad_replica_methods_for_UI(rest_client, auth_token):
+    """ REPLICA (REST): Test the listing of bad and suspicious replicas """
+    response = rest_client.get('/replicas/bad/states', headers=headers(auth(auth_token)))
+    assert response.status_code == 200
+    tot_files = []
+    for line in response.get_data(as_text=True).split('\n'):
+        if line != '':
+            tot_files.append(dumps(line))
+    nb_tot_files = len(tot_files)
 
-    def test_bad_replica_methods_for_UI(self):
-        """ REPLICA (REST): Test the listing of bad and suspicious replicas """
-        mw = []
-        headers1 = {'X-Rucio-Account': 'root', 'X-Rucio-Username': 'ddmlab', 'X-Rucio-Password': 'secret'}
-        headers1.update(self.vo_header)
-        result = TestApp(auth_app.wsgifunc(*mw)).get('/userpass', headers=headers1, expect_errors=True)
-        assert result.status == 200
-        token = str(result.header('X-Rucio-Auth-Token'))
-        headers2 = {'X-Rucio-Auth-Token': str(token)}
+    data = {'state': 'B'}
+    response = rest_client.get('/replicas/bad/states', headers=headers(auth(auth_token)), query_string=data)
+    assert response.status_code == 200
+    tot_bad_files = []
+    for line in response.get_data(as_text=True).split('\n'):
+        if line != '':
+            tot_bad_files.append(dumps(line))
+    nb_tot_bad_files1 = len(tot_bad_files)
 
-        data = dumps({})
-        result = TestApp(rep_app.wsgifunc(*mw)).get('/bad/states', headers=headers2, params=data, expect_errors=True)
-        assert result.status == 200
-        tot_files = []
-        for line in result.body.decode().split('\n'):
-            if line != '':
-                tot_files.append(dumps(line))
-        nb_tot_files = len(tot_files)
+    data = {'state': 'S', 'list_pfns': 'True'}
+    response = rest_client.get('/replicas/bad/states', headers=headers(auth(auth_token)), query_string=data)
+    assert response.status_code == 200
+    tot_suspicious_files = []
+    for line in response.get_data(as_text=True).split('\n'):
+        if line != '':
+            tot_suspicious_files.append(dumps(line))
+    nb_tot_suspicious_files = len(tot_suspicious_files)
 
-        data = dumps({'state': 'B'})
-        result = TestApp(rep_app.wsgifunc(*mw)).get('/bad/states', headers=headers2, params=data, expect_errors=True)
-        assert result.status == 200
-        tot_bad_files = []
-        for line in result.body.decode().split('\n'):
-            if line != '':
-                tot_bad_files.append(dumps(line))
-        nb_tot_bad_files1 = len(tot_bad_files)
+    data = {'state': 'T', 'list_pfns': 'True'}
+    response = rest_client.get('/replicas/bad/states', headers=headers(auth(auth_token)), query_string=data)
+    assert response.status_code == 200
+    tot_temporary_unavailable_files = []
+    for line in response.get_data(as_text=True).split('\n'):
+        if line != '':
+            tot_temporary_unavailable_files.append(dumps(line))
+    nb_tot_temporary_unavailable_files = len(tot_temporary_unavailable_files)
 
-        data = dumps({'state': 'S', 'list_pfns': 'True'})
-        result = TestApp(rep_app.wsgifunc(*mw)).get('/bad/states', headers=headers2, params=data, expect_errors=True)
-        assert result.status == 200
-        tot_suspicious_files = []
-        for line in result.body.decode().split('\n'):
-            if line != '':
-                tot_suspicious_files.append(dumps(line))
-        nb_tot_suspicious_files = len(tot_suspicious_files)
+    assert nb_tot_files == nb_tot_bad_files1 + nb_tot_suspicious_files + nb_tot_temporary_unavailable_files
 
-        data = dumps({'state': 'T', 'list_pfns': 'True'})
-        result = TestApp(rep_app.wsgifunc(*mw)).get('/bad/states', headers=headers2, params=data, expect_errors=True)
-        assert result.status == 200
-        tot_temporary_unavailable_files = []
-        for line in result.body.decode().split('\n'):
-            if line != '':
-                tot_temporary_unavailable_files.append(dumps(line))
-        nb_tot_temporary_unavailable_files = len(tot_temporary_unavailable_files)
+    tomorrow = datetime.utcnow() + timedelta(days=1)
+    data = {'state': 'B', 'younger_than': tomorrow.isoformat()}
+    response = rest_client.get('/replicas/bad/states', headers=headers(auth(auth_token)), query_string=data)
+    assert response.status_code == 200
+    tot_bad_files = []
+    for line in response.get_data(as_text=True).split('\n'):
+        if line != '':
+            tot_bad_files.append(dumps(line))
+    nb_tot_bad_files = len(tot_bad_files)
+    assert nb_tot_bad_files == 0
 
-        assert nb_tot_files == nb_tot_bad_files1 + nb_tot_suspicious_files + nb_tot_temporary_unavailable_files
+    response = rest_client.get('/replicas/bad/summary', headers=headers(auth(auth_token)))
+    assert response.status_code == 200
+    nb_tot_bad_files2 = 0
+    for line in response.get_data(as_text=True).split('\n'):
+        if line != '':
+            line = loads(line)
+            nb_tot_bad_files2 += int(line.get('BAD', 0))
+    assert nb_tot_bad_files1 == nb_tot_bad_files2
 
-        tomorrow = datetime.utcnow() + timedelta(days=1)
-        data = dumps({'state': 'B', 'younger_than': tomorrow.isoformat()})
-        result = TestApp(rep_app.wsgifunc(*mw)).get('/bad/states', headers=headers2, params=data, expect_errors=True)
-        assert result.status == 200
-        tot_bad_files = []
-        for line in result.body.decode().split('\n'):
-            if line != '':
-                tot_bad_files.append(dumps(line))
-        nb_tot_bad_files = len(tot_bad_files)
-        assert nb_tot_bad_files == 0
 
-        data = dumps({})
-        result = TestApp(rep_app.wsgifunc(*mw)).get('/bad/summary', headers=headers2, params=data, expect_errors=True)
-        assert result.status == 200
-        nb_tot_bad_files2 = 0
-        for line in result.body.decode().split('\n'):
-            if line != '':
-                line = loads(line)
-                nb_tot_bad_files2 += int(line.get('BAD', 0))
-        assert nb_tot_bad_files1 == nb_tot_bad_files2
+def test_list_replicas_content_type(replica_client, rest_client, auth_token):
+    """ REPLICA (REST): send a GET to list replicas with specific ACCEPT header."""
+    scope = 'mock'
+    name = 'file_%s' % generate_uuid()
+    files1 = [{'scope': scope, 'name': name, 'bytes': 1, 'adler32': '0cc737eb', 'meta': {'events': 10}}]
+    replica_client.add_replicas(rse='MOCK', files=files1)
 
-    def test_list_replicas_content_type(self):
-        """ REPLICA (REST): send a GET to list replicas with specific ACCEPT header."""
-        mw = []
-        account = 'root'
-        headers1 = {'X-Rucio-Account': account, 'X-Rucio-Username': 'ddmlab', 'X-Rucio-Password': 'secret'}
-        headers1.update(self.vo_header)
-        res1 = TestApp(auth_app.wsgifunc(*mw)).get('/userpass', headers=headers1, expect_errors=True)
-        assert res1.status == 200
-        token = str(res1.header('X-Rucio-Auth-Token'))
-        scope = 'mock'
-        name = 'file_%s' % generate_uuid()
-        files1 = [{'scope': scope, 'name': name, 'bytes': 1, 'adler32': '0cc737eb', 'meta': {'events': 10}}]
-        self.replica_client.add_replicas(rse='MOCK', files=files1)
+    # unsupported requested content type
+    response = rest_client.get('/replicas/%s/%s' % (scope, name), headers=headers(auth(auth_token), [('Accept', 'application/unsupported')]))
+    assert response.status_code == 406
 
-        # unsupported requested content type
-        headers = {'X-Rucio-Auth-Token': str(token), 'Accept': 'application/unsupported'}
-        res = TestApp(rep_app.wsgifunc(*mw)).get('/%s/%s' % (scope, name), headers=headers, expect_errors=True)
-        assert res.status == 406
+    # content type json stream
+    response = rest_client.get('/replicas/%s/%s' % (scope, name), headers=headers(auth(auth_token), [('Accept', 'application/x-json-stream')]))
+    assert [header[1] for header in response.headers if header[0] == 'Content-Type'][0] == 'application/x-json-stream'
 
-        # content type json stream
-        headers = {'X-Rucio-Auth-Token': str(token), 'Accept': 'application/x-json-stream'}
-        res = TestApp(rep_app.wsgifunc(*mw)).get('/%s/%s' % (scope, name), headers=headers, expect_errors=True)
-        assert [header[1] for header in res.headers if header[0] == 'Content-Type'][0] == 'application/x-json-stream'
+    # content type metalink4
+    response = rest_client.get('/replicas/%s/%s' % (scope, name), headers=headers(auth(auth_token), [('Accept', 'application/metalink4+xml')]))
+    assert [header[1] for header in response.headers if header[0] == 'Content-Type'][0] == 'application/metalink4+xml'
 
-        # content type metalink4
-        headers = {'X-Rucio-Auth-Token': str(token), 'Accept': 'application/metalink4+xml'}
-        res = TestApp(rep_app.wsgifunc(*mw)).get('/%s/%s' % (scope, name), headers=headers, expect_errors=True)
-        assert [header[1] for header in res.headers if header[0] == 'Content-Type'][0] == 'application/metalink4+xml'
+    # no requested content type
+    response = rest_client.get('/replicas/%s/%s' % (scope, name), headers=headers(auth(auth_token)))
+    assert [header[1] for header in response.headers if header[0] == 'Content-Type'][0] == 'application/x-json-stream'
 
-        # no requested content type
-        headers = {'X-Rucio-Auth-Token': str(token)}
-        res = TestApp(rep_app.wsgifunc(*mw)).get('/%s/%s' % (scope, name), headers=headers, expect_errors=True)
-        assert [header[1] for header in res.headers if header[0] == 'Content-Type'][0] == 'application/x-json-stream'
+    # all content types
+    response = rest_client.get('/replicas/%s/%s' % (scope, name), headers=headers(auth(auth_token), [('Accept', '*/*')]))
+    assert [header[1] for header in response.headers if header[0] == 'Content-Type'][0] == 'application/x-json-stream'
 
-        # all content types
-        headers = {'X-Rucio-Auth-Token': str(token), 'Accept': '*/*'}
-        res = TestApp(rep_app.wsgifunc(*mw)).get('/%s/%s' % (scope, name), headers=headers, expect_errors=True)
-        assert [header[1] for header in res.headers if header[0] == 'Content-Type'][0] == 'application/x-json-stream'
+    # multiple content types
+    response = rest_client.get('/replicas/%s/%s' % (scope, name), headers=headers(auth(auth_token), [('Accept', 'application/unsupported, application/x-json-stream')]))
+    assert [header[1] for header in response.headers if header[0] == 'Content-Type'][0] == 'application/x-json-stream'
 
-        # multiple content types
-        headers = {'X-Rucio-Auth-Token': str(token), 'Accept': 'application/unsupported, application/x-json-stream'}
-        res = TestApp(rep_app.wsgifunc(*mw)).get('/%s/%s' % (scope, name), headers=headers, expect_errors=True)
-        assert [header[1] for header in res.headers if header[0] == 'Content-Type'][0] == 'application/x-json-stream'
-        headers = {'X-Rucio-Auth-Token': str(token), 'Accept': 'application/unsupported, */*;q=0.8'}
-        res = TestApp(rep_app.wsgifunc(*mw)).get('/%s/%s' % (scope, name), headers=headers, expect_errors=True)
-        assert [header[1] for header in res.headers if header[0] == 'Content-Type'][0] == 'application/x-json-stream'
+    response = rest_client.get('/replicas/%s/%s' % (scope, name), headers=headers(auth(auth_token), [('Accept', 'application/unsupported, */*;q=0.8')]))
+    assert [header[1] for header in response.headers if header[0] == 'Content-Type'][0] == 'application/x-json-stream'
 
-    def test_add_list_replicas(self):
-        """ REPLICA (CLIENT): Add, change state and list file replicas """
-        tmp_scope = 'mock'
-        nbfiles = 5
 
-        files1 = [{'scope': tmp_scope, 'name': 'file_%s' % generate_uuid(), 'bytes': 1, 'adler32': '0cc737eb', 'meta': {'events': 10}} for _ in range(nbfiles)]
-        self.replica_client.add_replicas(rse='MOCK', files=files1)
+def test_add_list_replicas(replica_client):
+    """ REPLICA (CLIENT): Add, change state and list file replicas """
+    tmp_scope = 'mock'
+    nbfiles = 5
 
-        files2 = [{'scope': tmp_scope, 'name': 'file_%s' % generate_uuid(), 'bytes': 1, 'adler32': '0cc737eb', 'meta': {'events': 10}} for _ in range(nbfiles)]
-        self.replica_client.add_replicas(rse='MOCK3', files=files2)
+    files1 = [{'scope': tmp_scope, 'name': 'file_%s' % generate_uuid(), 'bytes': 1, 'adler32': '0cc737eb', 'meta': {'events': 10}} for _ in range(nbfiles)]
+    replica_client.add_replicas(rse='MOCK', files=files1)
 
-        replicas = [r for r in self.replica_client.list_replicas(dids=[{'scope': i['scope'], 'name': i['name']} for i in files1])]
-        assert len(replicas) == len(files1)
+    files2 = [{'scope': tmp_scope, 'name': 'file_%s' % generate_uuid(), 'bytes': 1, 'adler32': '0cc737eb', 'meta': {'events': 10}} for _ in range(nbfiles)]
+    replica_client.add_replicas(rse='MOCK3', files=files2)
 
-        replicas = [r for r in self.replica_client.list_replicas(dids=[{'scope': i['scope'], 'name': i['name']} for i in files2], schemes=['file'])]
-        assert len(replicas) == 5
+    replicas = [r for r in replica_client.list_replicas(dids=[{'scope': i['scope'], 'name': i['name']} for i in files1])]
+    assert len(replicas) == len(files1)
 
-        replicas = [r for r in self.replica_client.list_replicas(dids=[{'scope': i['scope'], 'name': i['name']} for i in files2], schemes=['srm'])]
-        assert len(replicas) == 5
+    replicas = [r for r in replica_client.list_replicas(dids=[{'scope': i['scope'], 'name': i['name']} for i in files2], schemes=['file'])]
+    assert len(replicas) == 5
 
-        files3 = [{'scope': tmp_scope, 'name': 'file_%s' % generate_uuid(), 'bytes': 1, 'adler32': '0cc737eb', 'state': 'U', 'meta': {'events': 10}} for _ in range(nbfiles)]
-        self.replica_client.add_replicas(rse='MOCK3', files=files3)
-        replicas = [r for r in self.replica_client.list_replicas(dids=[{'scope': i['scope'], 'name': i['name']} for i in files3], schemes=['file'])]
-        for i in range(nbfiles):
-            assert replicas[i]['rses'] == {}
-        files4 = []
-        for file in files3:
-            file['state'] = 'A'
-            files4.append(file)
-        self.replica_client.update_replicas_states('MOCK3', files=files4)
-        replicas = [r for r in self.replica_client.list_replicas(dids=[{'scope': i['scope'], 'name': i['name']} for i in files3], schemes=['file'], unavailable=True)]
-        assert len(replicas) == 5
-        for i in range(nbfiles):
-            assert 'MOCK3' in replicas[i]['rses']
+    replicas = [r for r in replica_client.list_replicas(dids=[{'scope': i['scope'], 'name': i['name']} for i in files2], schemes=['srm'])]
+    assert len(replicas) == 5
 
-    def test_delete_replicas(self):
-        """ REPLICA (CLIENT): Add and delete file replicas """
-        tmp_scope = 'mock'
-        nbfiles = 5
-        files = [{'scope': tmp_scope, 'name': 'file_%s' % generate_uuid(), 'bytes': 1, 'adler32': '0cc737eb', 'meta': {'events': 10}} for _ in range(nbfiles)]
-        self.replica_client.add_replicas(rse='MOCK', files=files)
-        with pytest.raises(AccessDenied):
-            self.replica_client.delete_replicas(rse='MOCK', files=files)
+    files3 = [{'scope': tmp_scope, 'name': 'file_%s' % generate_uuid(), 'bytes': 1, 'adler32': '0cc737eb', 'state': 'U', 'meta': {'events': 10}} for _ in range(nbfiles)]
+    replica_client.add_replicas(rse='MOCK3', files=files3)
+    replicas = [r for r in replica_client.list_replicas(dids=[{'scope': i['scope'], 'name': i['name']} for i in files3], schemes=['file'])]
+    for i in range(nbfiles):
+        assert replicas[i]['rses'] == {}
+    files4 = []
+    for file in files3:
+        file['state'] = 'A'
+        files4.append(file)
+    replica_client.update_replicas_states('MOCK3', files=files4)
+    replicas = [r for r in replica_client.list_replicas(dids=[{'scope': i['scope'], 'name': i['name']} for i in files3], schemes=['file'], unavailable=True)]
+    assert len(replicas) == 5
+    for i in range(nbfiles):
+        assert 'MOCK3' in replicas[i]['rses']
 
-        # replicas = [r for r in self.replica_client.list_replicas(dids=[{'scope': i['scope'], 'name': i['name']} for i in files])]
-        # assert len(replicas) == 0
 
-    def test_add_temporary_unavailable_pfns(self):
-        """ REPLICA (CLIENT): Add temporary unavailable PFNs"""
-        tmp_scope = 'mock'
-        nbfiles = 5
-        # Adding replicas to deterministic RSE
-        files = [{'scope': tmp_scope, 'name': 'file_%s' % generate_uuid(), 'bytes': 1, 'adler32': '0cc737eb', 'meta': {'events': 10}} for _ in range(nbfiles)]
-        self.replica_client.add_replicas(rse='MOCK', files=files)
+def test_add_replica_scope_not_found(replica_client):
+    """ REPLICA (CLIENT): Add replica with missing scope """
+    files = [{'scope': 'nonexistingscope', 'name': 'file_%s' % generate_uuid(), 'bytes': 1, 'adler32': '0cc737eb'}]
+    with pytest.raises(ScopeNotFound):
+        replica_client.add_replicas(rse='MOCK', files=files)
 
-        # Listing replicas on deterministic RSE
-        list_rep = []
-        for replica in self.replica_client.list_replicas(dids=[{'scope': f['scope'], 'name': f['name']} for f in files], schemes=['srm'], unavailable=True):
-            pfn = list(replica['pfns'].keys())[0]
-            list_rep.append(pfn)
 
-        # Submit bad PFNs
-        now = datetime.utcnow()
-        reason_str = generate_uuid()
-        self.replica_client.add_bad_pfns(pfns=list_rep, reason=str(reason_str), state='TEMPORARY_UNAVAILABLE', expires_at=now.isoformat())
-        result = get_bad_pfns(limit=10000, thread=None, total_threads=None, session=None)
-        bad_pfns = {}
-        for res in result:
-            bad_pfns[res['pfn']] = (res['state'], res['reason'], res['expires_at'])
+def test_delete_replicas(replica_client):
+    """ REPLICA (CLIENT): Add and delete file replicas """
+    tmp_scope = 'mock'
+    nbfiles = 5
+    files = [{'scope': tmp_scope, 'name': 'file_%s' % generate_uuid(), 'bytes': 1, 'adler32': '0cc737eb', 'meta': {'events': 10}} for _ in range(nbfiles)]
+    replica_client.add_replicas(rse='MOCK', files=files)
+    with pytest.raises(AccessDenied):
+        replica_client.delete_replicas(rse='MOCK', files=files)
 
-        for pfn in list_rep:
-            pfn = str(clean_surls([pfn])[0])
-            assert pfn in bad_pfns
-            assert str(bad_pfns[pfn][0]) == 'TEMPORARY_UNAVAILABLE'
-            assert bad_pfns[pfn][1] == reason_str
+    # replicas = [r for r in replica_client.list_replicas(dids=[{'scope': i['scope'], 'name': i['name']} for i in files])]
+    # assert len(replicas) == 0
 
-        # Submit with wrong state
-        with pytest.raises(RucioException):
-            self.replica_client.add_bad_pfns(pfns=list_rep, reason=str(reason_str), state='BADSTATE', expires_at=now.isoformat())
 
-        # Run minos once
-        minos_run(threads=1, bulk=10000, once=True)
-        result = get_bad_pfns(limit=10000, thread=None, total_threads=None, session=None)
-        pfns = [res['pfn'] for res in result]
-        res_pfns = []
-        for replica in list_rep:
-            if replica in pfns:
-                res_pfns.append(replica)
-        assert res_pfns == []
+def test_add_temporary_unavailable_pfns(vo, replica_client):
+    """ REPLICA (CLIENT): Add temporary unavailable PFNs"""
+    tmp_scope = 'mock'
+    nbfiles = 5
+    # Adding replicas to deterministic RSE
+    files = [{'scope': tmp_scope, 'name': 'file_%s' % generate_uuid(), 'bytes': 1, 'adler32': '0cc737eb', 'meta': {'events': 10}} for _ in range(nbfiles)]
+    replica_client.add_replicas(rse='MOCK', files=files)
 
-        # Check the state in the replica table
-        for did in files:
-            rep = get_replicas_state(scope=InternalScope(did['scope'], **self.vo), name=did['name'])
-            assert str(list(rep.keys())[0]) == 'TEMPORARY_UNAVAILABLE'
+    # Listing replicas on deterministic RSE
+    list_rep = []
+    for replica in replica_client.list_replicas(dids=[{'scope': f['scope'], 'name': f['name']} for f in files], schemes=['srm'], unavailable=True):
+        pfn = list(replica['pfns'].keys())[0]
+        list_rep.append(pfn)
 
-        rep = []
-        for did in files:
-            did['state'] = ReplicaState.from_sym('TEMPORARY_UNAVAILABLE')
-            rep.append(did)
+    # Submit bad PFNs
+    now = datetime.utcnow()
+    reason_str = generate_uuid()
+    replica_client.add_bad_pfns(pfns=list_rep, reason=str(reason_str), state='TEMPORARY_UNAVAILABLE', expires_at=now.isoformat())
+    result = get_bad_pfns(limit=10000, thread=None, total_threads=None, session=None)
+    bad_pfns = {}
+    for res in result:
+        bad_pfns[res['pfn']] = (res['state'], res['reason'], res['expires_at'])
 
-        # Run the minos expiration
-        minos_temp_run(threads=1, once=True)
-        # Check the state in the replica table
-        for did in files:
-            rep = get_replicas_state(scope=InternalScope(did['scope'], **self.vo), name=did['name'])
-            assert str(list(rep.keys())[0]) == 'AVAILABLE'
+    for pfn in list_rep:
+        pfn = str(clean_surls([pfn])[0])
+        assert pfn in bad_pfns
+        assert str(bad_pfns[pfn][0]) == 'TEMPORARY_UNAVAILABLE'
+        assert bad_pfns[pfn][1] == reason_str
 
-    def test_set_tombstone(self):
-        """ REPLICA (CLIENT): set tombstone on replica """
-        # Set tombstone on one replica
-        rse = 'MOCK4'
-        rse_id = get_rse_id(rse=rse, **self.vo)
-        scope = InternalScope('mock', **self.vo)
-        user = InternalAccount('root', **self.vo)
-        name = generate_uuid()
-        add_replica(rse_id, scope, name, 4, user)
-        assert get_replica(rse_id, scope, name)['tombstone'] is None
-        self.replica_client.set_tombstone([{'rse': rse, 'scope': scope.external, 'name': name}])
-        assert get_replica(rse_id, scope, name)['tombstone'] == OBSOLETE
+    # Submit with wrong state
+    with pytest.raises(RucioException):
+        replica_client.add_bad_pfns(pfns=list_rep, reason=str(reason_str), state='BADSTATE', expires_at=now.isoformat())
 
-        # Set tombstone on locked replica
-        name = generate_uuid()
-        add_replica(rse_id, scope, name, 4, user)
-        RuleClient().add_replication_rule([{'name': name, 'scope': scope.external}], 1, rse, locked=True)
-        with pytest.raises(ReplicaIsLocked):
-            self.replica_client.set_tombstone([{'rse': rse, 'scope': scope.external, 'name': name}])
+    # Run minos once
+    minos_run(threads=1, bulk=10000, once=True)
+    result = get_bad_pfns(limit=10000, thread=None, total_threads=None, session=None)
+    pfns = [res['pfn'] for res in result]
+    res_pfns = []
+    for replica in list_rep:
+        if replica in pfns:
+            res_pfns.append(replica)
+    assert res_pfns == []
 
-        # Set tombstone on not found replica
-        name = generate_uuid()
-        with pytest.raises(ReplicaNotFound):
-            self.replica_client.set_tombstone([{'rse': rse, 'scope': scope.external, 'name': name}])
+    # Check the state in the replica table
+    for did in files:
+        rep = get_replicas_state(scope=InternalScope(did['scope'], vo=vo), name=did['name'])
+        assert str(list(rep.keys())[0]) == 'TEMPORARY_UNAVAILABLE'
+
+    rep = []
+    for did in files:
+        did['state'] = ReplicaState.from_sym('TEMPORARY_UNAVAILABLE')
+        rep.append(did)
+
+    # Run the minos expiration
+    minos_temp_run(threads=1, once=True)
+    # Check the state in the replica table
+    for did in files:
+        rep = get_replicas_state(scope=InternalScope(did['scope'], vo=vo), name=did['name'])
+        assert str(list(rep.keys())[0]) == 'AVAILABLE'
+
+
+def test_set_tombstone2(vo, replica_client):
+    """ REPLICA (CLIENT): set tombstone on replica """
+    # Set tombstone on one replica
+    rse = 'MOCK4'
+    rse_id = get_rse_id(rse=rse, vo=vo)
+    scope = InternalScope('mock', vo=vo)
+    user = InternalAccount('root', vo=vo)
+    name = generate_uuid()
+    add_replica(rse_id, scope, name, 4, user)
+    assert get_replica(rse_id, scope, name)['tombstone'] is None
+    replica_client.set_tombstone([{'rse': rse, 'scope': scope.external, 'name': name}])
+    assert get_replica(rse_id, scope, name)['tombstone'] == OBSOLETE
+
+    # Set tombstone on locked replica
+    name = generate_uuid()
+    add_replica(rse_id, scope, name, 4, user)
+    RuleClient().add_replication_rule([{'name': name, 'scope': scope.external}], 1, rse, locked=True)
+    with pytest.raises(ReplicaIsLocked):
+        replica_client.set_tombstone([{'rse': rse, 'scope': scope.external, 'name': name}])
+
+    # Set tombstone on not found replica
+    name = generate_uuid()
+    with pytest.raises(ReplicaNotFound):
+        replica_client.set_tombstone([{'rse': rse, 'scope': scope.external, 'name': name}])
 
 
 class TestReplicaMetalink(unittest.TestCase):
